const St = imports.gi.St;
const Mainloop = imports.mainloop;
const GLib = imports.gi.GLib;
const Lang = imports.lang;

const Main = imports.ui.main;
const Panel = Main.panel
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const NMC = imports.gi.NMClient;
const device_type_map = ['none', 'wired', 'wireless', 'gsm', 'cdma'];

const timer=1000;

function LayoutMenuItem() {
    this._init.apply(this, arguments);
}

LayoutMenuItem.prototype = {
    __proto__: PopupMenu.PopupBaseMenuItem.prototype,

    _init: function(device, icon) {
        PopupMenu.PopupBaseMenuItem.prototype._init.call(this);
				this._icon = icon;
        this._device = device;
				this._device_title = new St.Label ({ text: device });
      	this._down_label = new St.Label ({ text: ""});
				this._up_label = new St.Label ({text: ""});
				this._down_label.set_width (65);
				this._up_label.set_width (65);
				if (this._icon != null)
					this.addActor(this._icon);
				else
					this.addActor (new St.Label ());
        this.addActor(this._device_title);
        this.addActor(this._down_label);
				this.addActor(this._up_label);
    },

		update: function(up, down) {
			this._down_label.set_text(down);
			this._up_label.set_text(up);		
		},

};

function NetSpeedStatusIcon()
{
	this._init.apply (this, arguments);
}

NetSpeedStatusIcon.prototype = {
    __proto__: PanelMenu.Button.prototype,

		_init: function () {
	    PanelMenu.Button.prototype._init.call(this, 0.0);

			this._box = new St.BoxLayout();
			this._icon = this._get_icon ("");
			this._text = new St.Label({ text: 'Unknown', style_class: 'ns-label'});
			this._text.set_width (65);
			this._box.add_actor (this._text);
			this._box.add_actor (this._icon);
			this.actor.add_actor (this._box);
			this._menu_title = new LayoutMenuItem("Device", null);
			this._menu_title.update ("Up", "Down");
			this.menu.addMenuItem (this._menu_title);
			this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
			this._layouts = new Array();
	},

	/*********************** get device icon by type ******************************/
	_get_icon: function (devicetype) {
		let _iconname = 'network-transmit-receive';
		if (devicetype == 'wireless')
				_iconname = 'network-wireless-signal-excellent';
		else if (devicetype == 'wired')
				_iconname = 'network-wired-symbolic';
		else if (devicetype == 'gsm')
				_iconname = 'network-cellular-edge-symbolic';
		else if (devicetype == 'cdma')
				_iconname = 'network-cellular-umts-symbolic'; 
		else if (devicetype == 'none')
				_iconname = 'network-offline';
		//else if (devicetype = 'wimax')
		//else if (devicetype = 'modem')
			
   return new St.Icon(
				{ icon_name: _iconname,
        	icon_type: St.IconType.SYMBOLIC,
					icon_size: 16,
				});

	},
	
	/****************** Set the Label ****************************************/
	set_text: function (text) 
	{
		this._text.set_text(text);
	},

	/**************** Destroy each element in array ***************************/
	_destroy_array: function (array) {
		for (let i = 0; i < array.length; ++i)
			array[i].destroy();
	},


	/************************* create the menu  ******************************************/
	create_menu: function (devices, types) {
		this._destroy_array (this._layouts);
		this._layouts = new Array();
		for (let i = 0; i < devices.length; ++i)	
		{	
			let icon = this._get_icon (types[i]);
			let layout = new LayoutMenuItem(devices[i], icon);
			this._layouts.push (layout);
			this.menu.addMenuItem (layout);
		}
	},

	/********************************** update each devices speed *************************************/
	update_speeds : function (speeds) {
		for (let i = 0; i < speeds.length; ++i)
		{
			this._layouts[i].update (speeds[i][1], speeds[i][0]);
		}
	}, 

};

function NetSpeed()
{
	this._init.apply (this, arguments);
}

NetSpeed.prototype = {

	/********************* check if there was a change in devices ******************/
	_check_devices: function() {
		if (this._devices.length != this._olddevices.length)
			return 0;
		for (let i = 0; i < this._devices.length; ++i)
			if (this._devices[i] != this._olddevices[i])
				return 0;
		return 1;
	},

	_dump_array: function (array) {
		 global.log (JSON.stringify (array));
	},

	/********** get device type maybe from netwrok-manager **************************/
	_get_device_type: function (device) {
/*
		let devfile="/sys/class/net/" + dev + "/wireless";
		if (GLib.file_test (devfile, GLib.FileTest.EXISTS))
			return 'wireless';
		else
			return 'wired';
	*/
		let devices =	this._client.get_devices() || [ ];

		for each (let dev in devices) 
			if (dev.interface == device) 
				if (dev.device_type < device_type_map.length)
					return device_type_map[dev.device_type];
				else
					return 'none';

		return 'none';
	},
	
	_round3: function(amount)
	{
		if (amount < 10)
			return Math.round(amount * 100) / 100;
		else if (amount < 100)
			return Math.round(amount * 10) / 10;
		else 
			return Math.round(amount);
	},

	/************************* speed to user-friendly string ***************************************/
	_speed_to_string: function (amount)	{
		let _res = "0"; // 1M=1024K, 1MB/s=1000MB/s
		if (amount > 1000000)
			_res =  this._round3(amount/1000000).toString().concat('GB/s');
		if (amount > 1000) 
			_res =  this._round3(amount/1000).toString().concat('MB/s');
		else if (amount > 1)
			_res =  this._round3(amount).toString().concat('KB/s');
		else if (amount > 0)
			_res =  this._round3(amount*1000).toString().concat('B/s');
		return _res;
	},


	/*********************************** Update the Label  **************************************************/
	_set_text: function(text) {
		this._status_icon.set_text (text);
	},

	/*********************************** Ask to update the speeds for each device *****************************************/
	_update_speeds: function(){
		this._status_icon.update_speeds (this._speeds);
	},

	/************************************ Ask to create menu for each device ***********************************/
	_create_menu: function(){
		let types = new Array();
		for (let i = 0; i < this._devices.length; ++i)
			types.push(this._get_device_type (this._devices[i]));
		this._status_icon.create_menu (this._devices, types);
	},


	/******************************** Update invoked by timer ******************************/
	_update : function ()
	{
		let flines = GLib.file_get_contents('/proc/net/dev');  // Read the file
		let nlines = ("" + flines[1]).split("\n"); // Break to lines

		let up=0; // set initial
		let down=0;
		this._oldvalues = this._values;
		this._values = new Array();
		this._speeds = new Array();
		this._olddevices = this._devices;
		this._devices = new Array();
	
		
		let time = GLib.get_monotonic_time() / 1000; // current time
		let delta = time - this._last_time;
		this._last_time = time;
		for(let i = 3; i < nlines.length - 1 ; ++i) 
		{ //first 2 lines are for header	
			let line =  nlines[i].replace(/ +/g, " ").replace(/^ */g, "");
			let params = line.split (" ");
			//this._dump_array (params);
			// So store up/down values
			this._values.push ([parseInt(params[1]), parseInt(params[9])]);
			this._devices.push (params[0].replace(":",""));
		}
		var total = 0;
		if (this._check_devices () == 1)
		{
			for (let i = 0; i < this._values.length; ++i)
			{
				let _up = this._values[i][0] - this._oldvalues[i][0];
				let _down = this._values[i][1] - this._oldvalues[i][1];
				this._speeds.push([
					this._speed_to_string(_up / delta), // Download
					this._speed_to_string(_down / delta) // Upload
				]);
				total += _down + _up; 
			}
			this._set_text (this._speed_to_string(total / delta));
			this._update_speeds ();
		}
		else
		{
			this._create_menu();
		}
		return true;
	},

	/************************************* Constructor *************************************************/
	_init : function () {
			this._client = NMC.Client.new();
	},
	
	
	enable: function() {
		this._last_up = 0; // size of upload in previous snapshot
		this._last_down = 0; // size of download in previous snapshot
		this._last_time = 0; // time of the latest snapshot
		this._device_count = 0;

		this._values = new Array();
		this._devices = new Array();

		this._status_icon = new NetSpeedStatusIcon ();
		Panel.addToStatusArea('netspeed', this._status_icon, 0);
		Panel.__netspeed = this;
		this._timerid = Mainloop.timeout_add(timer, Lang.bind(this, this._update));
	},

	disable: function () {
    Main.panel._statusArea['netspeed'].destroy();
		if (this._timerid != 0)
		{
			Mainloop.source_remove(this._timerid);
			this._timerid = 0;
		}
	},
}


// Put your extension initialization code here
function init() {
	return new NetSpeed;
}

// vim: ts=2 sw=2
